package sun.font;

import java.awt.font.GlyphJustificationInfo;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.util.Map;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;
import java.awt.Font;

class ExtendedTextSourceLabel extends ExtendedTextLabel implements Decoration.Label
{
    TextSource source;
    private Decoration decorator;
    private Font font;
    private AffineTransform baseTX;
    private CoreMetrics cm;
    Rectangle2D lb;
    Rectangle2D ab;
    Rectangle2D vb;
    Rectangle2D ib;
    StandardGlyphVector gv;
    float[] charinfo;
    private static final int posx = 0;
    private static final int posy = 1;
    private static final int advx = 2;
    private static final int advy = 3;
    private static final int visx = 4;
    private static final int visy = 5;
    private static final int visw = 6;
    private static final int vish = 7;
    private static final int numvals = 8;
    
    public ExtendedTextSourceLabel(TextSource source, Decoration decorator) {
        this.source = source;
        this.decorator = decorator;
        this.finishInit();
    }
    
    public ExtendedTextSourceLabel(TextSource source, ExtendedTextSourceLabel extendedTextSourceLabel, int n) {
        this.source = source;
        this.decorator = extendedTextSourceLabel.decorator;
        this.finishInit();
    }
    
    private void finishInit() {
        this.font = this.source.getFont();
        Map<TextAttribute, Object> attributes = this.font.getAttributes();
        this.baseTX = AttributeValues.getBaselineTransform(attributes);
        if (this.baseTX == null) {
            this.cm = this.source.getCoreMetrics();
        }
        else {
            AffineTransform charTransform = AttributeValues.getCharTransform(attributes);
            if (charTransform == null) {
                charTransform = new AffineTransform();
            }
            this.font = this.font.deriveFont(charTransform);
            this.cm = CoreMetrics.get(this.font.getLineMetrics(this.source.getChars(), this.source.getStart(), this.source.getStart() + this.source.getLength(), this.source.getFRC()));
        }
    }
    
    @Override
    public Rectangle2D getLogicalBounds() {
        return this.getLogicalBounds(0.0f, 0.0f);
    }
    
    @Override
    public Rectangle2D getLogicalBounds(float n, float n2) {
        if (this.lb == null) {
            this.lb = this.createLogicalBounds();
        }
        return new Rectangle2D.Float((float)(this.lb.getX() + (double)n), (float)(this.lb.getY() + (double)n2), (float)this.lb.getWidth(), (float)this.lb.getHeight());
    }
    
    @Override
    public float getAdvance() {
        if (this.lb == null) {
            this.lb = this.createLogicalBounds();
        }
        return (float)this.lb.getWidth();
    }
    
    @Override
    public Rectangle2D getVisualBounds(float n, float n2) {
        if (this.vb == null) {
            this.vb = this.decorator.getVisualBounds(this);
        }
        return new Rectangle2D.Float((float)(this.vb.getX() + (double)n), (float)(this.vb.getY() + (double)n2), (float)this.vb.getWidth(), (float)this.vb.getHeight());
    }
    
    @Override
    public Rectangle2D getAlignBounds(float n, float n2) {
        if (this.ab == null) {
            this.ab = this.createAlignBounds();
        }
        return new Rectangle2D.Float((float)(this.ab.getX() + (double)n), (float)(this.ab.getY() + (double)n2), (float)this.ab.getWidth(), (float)this.ab.getHeight());
    }
    
    @Override
    public Rectangle2D getItalicBounds(float n, float n2) {
        if (this.ib == null) {
            this.ib = this.createItalicBounds();
        }
        return new Rectangle2D.Float((float)(this.ib.getX() + (double)n), (float)(this.ib.getY() + (double)n2), (float)this.ib.getWidth(), (float)this.ib.getHeight());
    }
    
    @Override
    public Rectangle getPixelBounds(FontRenderContext fontRenderContext, float n, float n2) {
        return this.getGV().getPixelBounds(fontRenderContext, n, n2);
    }
    
    @Override
    public boolean isSimple() {
        return this.decorator == Decoration.getPlainDecoration() && this.baseTX == null;
    }
    
    @Override
    public AffineTransform getBaselineTransform() {
        return this.baseTX;
    }
    
    @Override
    public Shape handleGetOutline(float n, float n2) {
        return this.getGV().getOutline(n, n2);
    }
    
    @Override
    public Shape getOutline(float n, float n2) {
        return this.decorator.getOutline(this, n, n2);
    }
    
    @Override
    public void handleDraw(Graphics2D graphics2D, float n, float n2) {
        graphics2D.drawGlyphVector(this.getGV(), n, n2);
    }
    
    @Override
    public void draw(Graphics2D graphics2D, float n, float n2) {
        this.decorator.drawTextAndDecorations(this, graphics2D, n, n2);
    }
    
    protected Rectangle2D createLogicalBounds() {
        return this.getGV().getLogicalBounds();
    }
    
    @Override
    public Rectangle2D handleGetVisualBounds() {
        return this.getGV().getVisualBounds();
    }
    
    protected Rectangle2D createAlignBounds() {
        float[] charinfo = this.getCharinfo();
        float max = 0.0f;
        float n = -this.cm.ascent;
        float n2 = 0.0f;
        float n3 = this.cm.ascent + this.cm.descent;
        if (this.charinfo == null || this.charinfo.length == 0) {
            return new Rectangle2D.Float(max, n, n2, n3);
        }
        int n4 = (this.source.getLayoutFlags() & 0x8) == 0x0;
        int n5 = charinfo.length - 8;
        if (!(n4 == 0)) {
            while (n5 > 0 && charinfo[n5 + 6] == 0.0f) {
                n5 -= 8;
            }
        }
        if (n5 >= 0) {
            int n6;
            for (n6 = 0; n6 < n5 && (charinfo[n6 + 2] == 0.0f || (n4 == 0 && charinfo[n6 + 6] == 0.0f)); n6 += 8) {}
            max = Math.max(0.0f, charinfo[n6 + 0]);
            n2 = charinfo[n5 + 0] + charinfo[n5 + 2] - max;
        }
        return new Rectangle2D.Float(max, n, n2, n3);
    }
    
    public Rectangle2D createItalicBounds() {
        float italicAngle = this.cm.italicAngle;
        Rectangle2D logicalBounds = this.getLogicalBounds();
        float x = (float)logicalBounds.getMinX();
        float y = -this.cm.ascent;
        float n = (float)logicalBounds.getMaxX();
        float descent = this.cm.descent;
        if (italicAngle != 0.0f) {
            if (italicAngle > 0.0f) {
                x -= italicAngle * (descent - this.cm.ssOffset);
                n -= italicAngle * (y - this.cm.ssOffset);
            }
            else {
                x -= italicAngle * (y - this.cm.ssOffset);
                n -= italicAngle * (descent - this.cm.ssOffset);
            }
        }
        return new Rectangle2D.Float(x, y, n - x, descent - y);
    }
    
    private final StandardGlyphVector getGV() {
        if (this.gv == null) {
            this.gv = this.createGV();
        }
        return this.gv;
    }
    
    protected StandardGlyphVector createGV() {
        FontRenderContext frc = this.source.getFRC();
        int layoutFlags = this.source.getLayoutFlags();
        char[] chars = this.source.getChars();
        int start = this.source.getStart();
        int length = this.source.getLength();
        GlyphLayout value = GlyphLayout.get(null);
        this.gv = value.layout(this.font, frc, chars, start, length, layoutFlags, null);
        GlyphLayout.done(value);
        return this.gv;
    }
    
    @Override
    public int getNumCharacters() {
        return this.source.getLength();
    }
    
    @Override
    public CoreMetrics getCoreMetrics() {
        return this.cm;
    }
    
    @Override
    public float getCharX(int n) {
        this.validate(n);
        float[] charinfo = this.getCharinfo();
        int n2 = this.l2v(n) * 8 + 0;
        if (charinfo == null || n2 >= charinfo.length) {
            return 0.0f;
        }
        return charinfo[n2];
    }
    
    @Override
    public float getCharY(int n) {
        this.validate(n);
        float[] charinfo = this.getCharinfo();
        int n2 = this.l2v(n) * 8 + 1;
        if (charinfo == null || n2 >= charinfo.length) {
            return 0.0f;
        }
        return charinfo[n2];
    }
    
    @Override
    public float getCharAdvance(int n) {
        this.validate(n);
        float[] charinfo = this.getCharinfo();
        int n2 = this.l2v(n) * 8 + 2;
        if (charinfo == null || n2 >= charinfo.length) {
            return 0.0f;
        }
        return charinfo[n2];
    }
    
    @Override
    public Rectangle2D handleGetCharVisualBounds(int n) {
        this.validate(n);
        float[] charinfo = this.getCharinfo();
        n = this.l2v(n) * 8;
        if (charinfo == null || n + 7 >= charinfo.length) {
            return new Rectangle2D.Float();
        }
        return new Rectangle2D.Float(charinfo[n + 4], charinfo[n + 5], charinfo[n + 6], charinfo[n + 7]);
    }
    
    @Override
    public Rectangle2D getCharVisualBounds(int n, float n2, float n3) {
        Rectangle2D charVisualBounds = this.decorator.getCharVisualBounds(this, n);
        if (n2 != 0.0f || n3 != 0.0f) {
            charVisualBounds.setRect(charVisualBounds.getX() + (double)n2, charVisualBounds.getY() + (double)n3, charVisualBounds.getWidth(), charVisualBounds.getHeight());
            return charVisualBounds;
        }
        return charVisualBounds;
    }
    
    private void validate(int n) {
        if (n < 0) {
            throw new IllegalArgumentException("index " + n + " < 0");
        }
        if (n >= this.source.getLength()) {
            throw new IllegalArgumentException("index " + n + " < " + this.source.getLength());
        }
        return;
    }
    
    @Override
    public int logicalToVisual(int n) {
        this.validate(n);
        return this.l2v(n);
    }
    
    @Override
    public int visualToLogical(int n) {
        this.validate(n);
        return this.v2l(n);
    }
    
    @Override
    public int getLineBreakIndex(int n, float n2) {
        float[] charinfo = this.getCharinfo();
        int length = this.source.getLength();
        --n;
        while (n2 >= 0.0f && ++n < length) {
            int n3 = this.l2v(n) * 8 + 2;
            if (n3 >= charinfo.length) {
                break;
            }
            n2 -= charinfo[n3];
        }
        return n;
    }
    
    @Override
    public float getAdvanceBetween(int n, int n2) {
        float n3 = 0.0f;
        float[] charinfo = this.getCharinfo();
        --n;
        while (++n < n2) {
            int n4 = this.l2v(n) * 8 + 2;
            if (n4 >= charinfo.length) {
                break;
            }
            n3 += charinfo[n4];
        }
        return n3;
    }
    
    @Override
    public boolean caretAtOffsetIsValid(int n) {
        if (n == 0 || n == this.source.getLength()) {
            return true;
        }
        int n2 = this.source.getChars()[this.source.getStart() + n];
        if (n2 == '\t' || n2 == '\n' || n2 == '\r') {
            return true;
        }
        int n3 = this.l2v(n) * 8 + 2;
        float[] charinfo = this.getCharinfo();
        return charinfo != null && n3 < charinfo.length && charinfo[n3] != 0.0f;
    }
    
    private final float[] getCharinfo() {
        if (this.charinfo == null) {
            this.charinfo = this.createCharinfo();
        }
        return this.charinfo;
    }
    
    protected float[] createCharinfo() {
        StandardGlyphVector gv = this.getGV();
        float[] glyphInfo = null;
        try {
            glyphInfo = gv.getGlyphInfo();
        }
        catch (Exception ex) {
            System.out.println(this.source);
        }
        int numGlyphs = gv.getNumGlyphs();
        if (numGlyphs == 0) {
            return glyphInfo;
        }
        int[] glyphCharIndices = gv.getGlyphCharIndices(0, numGlyphs, (int[])null);
        boolean n = false;
        if (!(n == 0)) {
            System.err.println("number of glyphs: " + numGlyphs);
            for (int i = 0; i < numGlyphs; ++i) {
                System.err.println("g: " + i + ", x: " + glyphInfo[i * 8 + 0] + ", a: " + glyphInfo[i * 8 + 2] + ", n: " + glyphCharIndices[i]);
            }
        }
        int n2 = glyphCharIndices[0];
        int n3 = 0;
        int n4 = 0;
        int n5 = 0;
        int n6 = 0;
        int j = 0;
        int n7 = 8;
        int n8 = 1;
        int n9 = (this.source.getLayoutFlags() & 0x1) == 0x0;
        if (n9 == 0) {
            int n10 = glyphCharIndices[numGlyphs - 1];
            n3 = 0;
            n4 = glyphInfo.length - 8;
            n5 = 0;
            n6 = glyphInfo.length - 8;
            j = numGlyphs - 1;
            numGlyphs = -1;
            n7 = -8;
            n8 = -1;
        }
        float min = 0.0f;
        float max = 0.0f;
        float min2 = 0.0f;
        float min3 = 0.0f;
        float max2 = 0.0f;
        float max3 = 0.0f;
        float n11 = 0.0f;
        boolean n12 = false;
        while (j != numGlyphs) {
            int n13 = 0;
            int n14;
            int k;
            for (n14 = 0, k = (k = glyphCharIndices[j]), j += n8, n6 += n7; j != numGlyphs && (glyphInfo[n6 + 2] == 0.0f || k != n3 || glyphCharIndices[j] <= k || k - k > n14); k = Math.min(k, glyphCharIndices[j]), k = Math.max(k, glyphCharIndices[j]), j += n8, n6 += n7) {
                if (n13 == 0) {
                    int n15 = n6 - n7;
                    min = glyphInfo[n15 + 0];
                    max = min + glyphInfo[n15 + 2];
                    min2 = glyphInfo[n15 + 4];
                    min3 = glyphInfo[n15 + 5];
                    max2 = min2 + glyphInfo[n15 + 6];
                    max3 = min3 + glyphInfo[n15 + 7];
                    n13 = 1;
                }
                ++n14;
                float n16 = glyphInfo[n6 + 2];
                if (n16 != 0.0f) {
                    float b = glyphInfo[n6 + 0];
                    min = Math.min(min, b);
                    max = Math.max(max, b + n16);
                }
                float n17 = glyphInfo[n6 + 6];
                if (n17 != 0.0f) {
                    float b2 = glyphInfo[n6 + 4];
                    float b3 = glyphInfo[n6 + 5];
                    min2 = Math.min(min2, b2);
                    min3 = Math.min(min3, b3);
                    max2 = Math.max(max2, b2 + n17);
                    max3 = Math.max(max3, b3 + glyphInfo[n6 + 7]);
                }
            }
            if (!(n == 0)) {
                System.out.println("minIndex = " + k + ", maxIndex = " + k);
            }
            n3 = k + 1;
            glyphInfo[n4 + 1] = n11;
            glyphInfo[n4 + 3] = 0.0f;
            if (n13 != 0) {
                glyphInfo[n4 + 2] = max - (glyphInfo[n4 + 0] = min);
                glyphInfo[n4 + 4] = min2;
                glyphInfo[n4 + 5] = min3;
                glyphInfo[n4 + 6] = max2 - min2;
                glyphInfo[n4 + 7] = max3 - min3;
                if (k - k < n14) {
                    n12 = true;
                }
                if (k < k) {
                    if (n9 == 0) {
                        max = min;
                    }
                    max2 -= min2;
                    max3 -= min3;
                    int l = n4 / 8;
                    while (k < k) {
                        ++k;
                        n5 += n8;
                        n4 += n7;
                        if ((n4 < 0 || n4 >= glyphInfo.length) && n) {
                            System.out.println("minIndex = " + k + ", maxIndex = " + k + ", cp = " + l);
                        }
                        glyphInfo[n4 + 0] = max;
                        glyphInfo[n4 + 1] = n11;
                        glyphInfo[n4 + 3] = (glyphInfo[n4 + 2] = 0.0f);
                        glyphInfo[n4 + 4] = min2;
                        glyphInfo[n4 + 5] = min3;
                        glyphInfo[n4 + 6] = max2;
                        glyphInfo[n4 + 7] = max3;
                    }
                }
            }
            else if (!(n12 == 0)) {
                int n18 = n6 - n7;
                glyphInfo[n4 + 0] = glyphInfo[n18 + 0];
                glyphInfo[n4 + 2] = glyphInfo[n18 + 2];
                glyphInfo[n4 + 4] = glyphInfo[n18 + 4];
                glyphInfo[n4 + 5] = glyphInfo[n18 + 5];
                glyphInfo[n4 + 6] = glyphInfo[n18 + 6];
                glyphInfo[n4 + 7] = glyphInfo[n18 + 7];
            }
            n4 += n7;
            n5 += n8;
        }
        if (n12 && n9 == 0) {
            int n19 = n4 - n7;
            System.arraycopy(glyphInfo, n19, glyphInfo, 0, glyphInfo.length - n19);
        }
        if (!(n == 0)) {
            char[] chars = this.source.getChars();
            int start = this.source.getStart();
            int length = this.source.getLength();
            System.out.println("char info for " + length + " characters");
            int n20 = 0;
            while (n20 < length * 8) {
                System.out.println(" ch: " + Integer.toHexString((int)chars[start + this.v2l(n20 / 8)]) + " x: " + glyphInfo[n20++] + " y: " + glyphInfo[n20++] + " xa: " + glyphInfo[n20++] + " ya: " + glyphInfo[n20++] + " l: " + glyphInfo[n20++] + " t: " + glyphInfo[n20++] + " w: " + glyphInfo[n20++] + " h: " + glyphInfo[n20++]);
            }
            return glyphInfo;
        }
        return glyphInfo;
    }
    
    protected int l2v(int n) {
        return ((this.source.getLayoutFlags() & 0x1) == 0x0) ? n : (this.source.getLength() - 1 - n);
    }
    
    protected int v2l(int n) {
        return ((this.source.getLayoutFlags() & 0x1) == 0x0) ? n : (this.source.getLength() - 1 - n);
    }
    
    @Override
    public TextLineComponent getSubset(int n, int n2, int n3) {
        return new ExtendedTextSourceLabel(this.source.getSubSource(n, n2 - n, n3), this.decorator);
    }
    
    @Override
    public String toString() {
        TextSource source = this.source;
        TextSource source2 = this.source;
        return source.toString(false);
    }
    
    @Override
    public int getNumJustificationInfos() {
        return this.getGV().getNumGlyphs();
    }
    
    @Override
    public void getJustificationInfos(GlyphJustificationInfo[] array, int n, int n2, int n3) {
        StandardGlyphVector gv = this.getGV();
        float[] charinfo = this.getCharinfo();
        float size2D = gv.getFont().getSize2D();
        GlyphJustificationInfo glyphJustificationInfo = new GlyphJustificationInfo(0.0f, false, 3, 0.0f, 0.0f, false, 3, 0.0f, 0.0f);
        GlyphJustificationInfo glyphJustificationInfo2 = new GlyphJustificationInfo(size2D, true, 1, 0.0f, size2D, true, 1, 0.0f, size2D / 4.0f);
        GlyphJustificationInfo glyphJustificationInfo3 = new GlyphJustificationInfo(size2D, true, 2, size2D, size2D, false, 3, 0.0f, 0.0f);
        char[] chars = this.source.getChars();
        int start = this.source.getStart();
        int numGlyphs = gv.getNumGlyphs();
        int n4 = 0;
        int n5 = (this.source.getLayoutFlags() & 0x1) == 0x0;
        if (n2 != 0 || n3 != this.source.getLength()) {
            if (!(n5 == 0)) {
                n4 = n2;
                numGlyphs = n3;
            }
            else {
                n4 = numGlyphs - n3;
                numGlyphs -= n2;
            }
        }
        GlyphJustificationInfo glyphJustificationInfo4;
        for (int i = 0; i < numGlyphs; array[n + i] = glyphJustificationInfo4, ++i) {
            glyphJustificationInfo4 = null;
            if (i >= n4 && i < numGlyphs) {
                if (charinfo[i * 8 + 2] == 0.0f) {
                    glyphJustificationInfo4 = glyphJustificationInfo;
                }
                else {
                    int ch = chars[start + this.v2l(i)];
                    if (Character.isWhitespace((char)ch)) {
                        glyphJustificationInfo4 = glyphJustificationInfo2;
                    }
                    else if ((ch >= '一' && ch < 'ꀀ') || (ch >= '가' && ch < 'ힰ') || (ch >= '豈' && ch < 'ﬀ')) {
                        glyphJustificationInfo4 = glyphJustificationInfo3;
                    }
                    else {
                        glyphJustificationInfo4 = glyphJustificationInfo;
                    }
                }
            }
        }
    }
    
    @Override
    public TextLineComponent applyJustificationDeltas(float[] array, int n, boolean[] array2) {
        float[] charinfo = (float[])(float[])this.getCharinfo().clone();
        array2[0] = false;
        StandardGlyphVector gv = (StandardGlyphVector)this.getGV().clone();
        float[] glyphPositions = gv.getGlyphPositions((float[])null);
        int numGlyphs = gv.getNumGlyphs();
        char[] chars = this.source.getChars();
        int start = this.source.getStart();
        float n2 = 0.0f;
        for (int i = 0; i < numGlyphs; ++i) {
            if (Character.isWhitespace(chars[start + this.v2l(i)])) {
                int n7 = i * 2;
                glyphPositions[n7] += n2;
                float n8 = array[n + i * 2] + array[n + i * 2 + 1];
                int n9 = i * 8 + 0;
                charinfo[n9] += n2;
                int n10 = i * 8 + 4;
                charinfo[n10] += n2;
                int n11 = i * 8 + 2;
                charinfo[n11] += n8;
                n2 += n8;
            }
            else {
                float n3 = n2 + array[n + i * 2];
                int n4 = i * 2;
                glyphPositions[n4] += n3;
                int n5 = i * 8 + 0;
                charinfo[n5] += n3;
                int n6 = i * 8 + 4;
                charinfo[n6] += n3;
                n2 = n3 + array[n + i * 2 + 1];
            }
        }
        int n12 = numGlyphs * 2;
        glyphPositions[n12] += n2;
        gv.setGlyphPositions(glyphPositions);
        ExtendedTextSourceLabel o = new ExtendedTextSourceLabel(this.source, this.decorator);
        o.gv = gv;
        o.charinfo = charinfo;
        return o;
    }
}
